
body {
  background: #FFFFFF
    url(../images/bg.png)
    repeat-x left top;
  font-family: Verdana, Arial, Helvetica, Geneva, sans-serif;
  color: #2A2828;
  font-size: 12px;
  margin-left: auto;
  margin-right: auto;
  padding: 0px;
}

/* Main layout  */
div.header {
  background: #FFFFFF url(../images/header.png) left top;
  height: 130px;
}

div.top_bar {
  background: #FFFFFF url(../images/top_bar_bg.png) left top;
  height: 35px;
}

/* Top-Bar */
div.top_left {
  background: url(../images/top_sep.png) no-repeat right top;
  float: left;
  padding: 5px 0px;
  margin-top: 2px;
  width: 20px;
  height: 20px;
}

div.top_link {
}

div.top_sep {
  background: url(../images/top_sep.png) no-repeat right top;
  margin-top: 2px;
  float: left;
  width: 2px;
  padding: 0px;
  height: 32px;
}

div.top_link a:hover, 
div.top_link a:active {
  background: url(../images/top_hover.png) repeat-x right top;
  margin-top: 2px;
  color: #BBBABA;
  font-size: 13;
  font-weight: normal;
}

div.top_link a:visited,
div.top_link a:link {
  color: #BBBABA;
  font-size: 13;
  font-weight: normal;
}

div.top_link a {
  display: block;
  float: left;
  margin-top: 2px;
  padding: 7px 15px 0px 15px;
  height: 32px;
}

/* Main-Box (Navi + Content) */
div.main_box {  
  background: url(../images/main_box_bg.png) repeat-y left top;  
}

div.main_top {
  background: url(../images/main_top.png) no-repeat left top;
  height: 300px;
}

div.main_bottom {
  background: url(../images/main_bottom.png) no-repeat left bottom;
  height: 17px;
}


div.main_content {
  position: relative;
  margin-top: -276px;
}

/* Content box (right)  */
div.content_box {  
  width: 710px;
  margin-left: 20px;
}

div.content_top {
  height: 2px;
  margin-left: -1px;
}

div.content_top_bg {
  background: url(../images/content_top.png) no-repeat left top;
}

div.content_bottom {
  height: 2px;
  margin-left: -1px;
}

div.content_bottom_bg {
  background: url(../images/content_bottom.png) no-repeat left bottom;
}

div.content_content {
    margin: 0px;
    padding: 0px;
    position: relative;
}

div.content_content_bg {
    background: #ffffff;
}

/* Navigation boxes (left) */
.navibox {
  width: 190px;
  padding-left: 0px;
  margin-left: 20px;
  background: url(../images/navi_box_bottom.png) no-repeat bottom right;
  margin-bottom: 4px;
}

.navihead {
  background: url(../images/navi_box_head.png) #fff no-repeat top right;
}

.navihead h5 {
    color: #494949;
    padding-bottom: 2px;
    padding-left: 15px;
    padding-top: 2px;
    margin-bottom: 0px;
    font-size: 13px;
    font-weight: normal;
}

.naviitems ul {
    list-style: none;
    margin-bottom: 0px;
    padding-bottom: 8px;
    padding-top: 5px;
    margin-left: 5px;
}

.naviitems li {
    color: #8e8e8e;
    margin-left: 10px;
}

.naviitems li.expanded {
    margin-left: 10px;
}

.naviitems li a:visited, .naviitems li a:link {
    color: #656565;
    display: block;
    font-weight: normal;
}

.naviitems li a:active, .naviitems li a:hover, .naviitems li strong {
    color: #a20000;
    background: url(../images/navi_link_hover.png) no-repeat left top;  ;
    display: block;
    width: 158px;
    margin-left: -2px;
    padding-left: 2px;
    font-weight: normal;
}

.copyright_footer {
    float: right;
    color: #8e8e8e;
    font-size: 9px;
    margin: 3px 25px -5px 0px;
}

.content_content {
    line-height: 170%;
}

div.teaser {
    background: url(../images/teaser_bg.png) left top no-repeat;
    width: 100%;
    color: #e7d3cb;
    font-size: 20px;
    padding: 13px 80px 0px;
    margin: -1px 2px 15px 2px;
    line-height: 150%;
    height: 68px;
}

.teaser .subtitle a.externalLink {
    background: none;
    color: #e7d3cb;
    font-size: 14px;    
}

.teaser a.externalLink:visited,.teaser a.externalLink:link {
    background: none;
    padding: 0;
    color: #e7d3cb;
}

.teaser a.externalLink:hover,.teaser a.externalLink:active  {
    background: none;
    padding: 0;
    color: #4a0501;
}

div.teaser b, div.teaser strong {
    color: #4a0501;
    font-weight: bold;
}

div.teaser .subtitle {
    font-size: 14px;
}

div.feature {
    padding-left: 10px;
}

div.feature_box {
    background: url(../images/feature_box_bottom.png) no-repeat bottom right;
    width: 271px;
    margin: 0px 30px 30px 0px;
    padding-left: 1px;
    min-height: 200px;
}

div.feature_box div.section {
    padding: 1px;
}

div.feature_icon {
    float: right;
    margin: 10px;
}

div.feature_icon_large {
    float: right; 
    margin: 0px 20px 20px 30px;
}


.feature_box h2 {
    background: url(../images/feature_box_head.png) #fff no-repeat top right;
    text-align: center;
    color: #fff;
    width: 100%;
    height: 24px;
    padding: 2px 0 0 1px;
    margin: -1px 0 0 0;
    font-size: 15px;
}

.feature_box p {
    margin: 10px;
    text-align: left;
}

/* ========================================================================================================  */
/* Common text attributes */

ul {
    list-style: square;
    margin-left: 5px;
}

.section ul {
    margin-left: 25px;
    margin-right: 25px;
    text-align: justify;
    margin-bottom: 5px;
}

.section ul li {
    margin-bottom: 2px;
    margin-top: 3px;
}

dd {
    margin: 0px 25px 10px 25px;
    text-align: justify;
}

p, .section > div {
    margin: 10px 25px;
    text-align: justify;
    position: relative;
}

code {
  font-size: 12px;
}

a {
  text-decoration: none;
}

a:link, a:visited {
  color:#a20000;
}

a:active, a:hover {
  background: #E9E9E9;
}

a:active.no-hover, a:hover.no-hover {
    background: none;
}

#legend li.externalLink {
  background: url(../images/external.png) left top no-repeat;
  padding-left: 18px;
}

a.externalLink, a.externalLink:link, a.externalLink:visited, a.externalLink:active, a.externalLink:hover {
  background: url(../images/external.png) right center no-repeat;
  padding-right: 15px;
}

a.externalLink:hover {
  background: #E9E9E9 url(../images/external.png) right center no-repeat;
}

#legend li.newWindow {
  background: url(../images/newwindow.png) left top no-repeat;
  padding-left: 18px;
}

a.newWindow, a.newWindow:link, a.newWindow:visited, a.newWindow:active, a.newWindow:hover {
  background: url(../images/newwindow.png) right center no-repeat;
  padding-right: 18px;
}

h2 {
    background: url(../images/h2_bg.png) no-repeat left top;
    display: block;
    font-size: 17px;
    padding-top: 5px;
    padding-left: 15px;
    margin-left: -4px;
    margin-top: -3px;
    height: 32px;
    color: #555;
    width: 710px;
    font-weight: normal;
}

h3 {
  padding: 4px 4px 4px 4px;
  color: #555;
  font-weight: normal;
  font-size: 17px;
  display: block;
  border-bottom: #555 1px solid;
}

h4 {
  padding: 4px 4px 4px 4px;
  color: #c20000;
  font-weight: normal;
  font-size: 17px;
}

h5 {
  padding: 4px 4px 4px 4px;
  font-size: 15px;
  color: #555;
}

#breadcrumbs {
  height: 6px;
  padding: 5px 10px 14px 205px;
}

* html #breadcrumbs {
  padding-bottom: 8px;
}

pre {
  border: 1px solid #999;
  background: #f5f5f5;
  margin: 1.5em 0px;
  padding: 8px;
  overflow: auto
}

div.source pre {
  border: 0;
}

div.source {
  border: 1px solid #999;
  margin: 1.5em 0px;
  background: #f5f5f5;
}

dt {
  padding: 4px 4px 0px 24px;
  color: #333333;
  font-weight: bold;
  font-size: 14px;
}
.subsectionTitle {
  font-size: 13px;
  font-weight: bold;
  color: #666;

}

.xright a:link, .xright a:visited, .xright a:active {
  color: #666;
}
.xright a:hover {
  color: #003300;
}
#breadcrumbs .xleft a:link, #breadcrumbs .xleft a:visited, #breadcrumbs .xleft a:active, #breadcrumbs .xleft a:hover {
  color: #FF9622;
  font-weight: bold;
}
#banner {
  height: 235px;
}

.subsection {
  margin-left: 3px;
  color: #333333;
}
.subsection p {
  font-size: 12px;
}
.errormark, .warningmark, .donemark, .infomark {
  background: url(../images/icon_error_sml.gif) no-repeat;
}

.warningmark {
  background-image: url(../images/icon_warning_sml.gif);
}

.donemark {
  background-image: url(../images/icon_success_sml.gif);
}

.infomark {
  background-image: url(../images/icon_info_sml.gif);
}

table.bodyTable { 
  font-size: 100%;
  margin: 10px 25px;
  width: 620px;
}

li table.bodyTable {
  width: 500px;
}

.bodyTable td, .bodyTable th {
  max-width: 470px;
}

.bodyTable th {
  text-align: left;
}

/* ========================== */

.str { 
    color: #061;
}

/* news date */

div.news-date {
    color: #C7C7C7;
    font-size: 10px;
    float: right;
    clear: both;
    margin-top: -40px;
}

/* ========================== */
/* Nabble */

div#nabbleforum {
    margin-top: -30px;
}

/* ========================== */
/* Idea.Informer */

.tegdiw_pot h2 {
  width: auto;
  background: none;
}

.bvnmrte {

}
