/*
 * Copyright 2009-2013 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * Basic layout
 */

html {
    margin: 0;
    height: 100%;
    min-height: 100%
}

body {
    margin: 0 auto;
    padding: 0;
    background-color: white;
    line-height: 150%;
    max-width: 960px;
}

body, td, div {
    font-family: 'DejaVu Sans', 'Lucida Grande', 'Verdana', sans-serif;
    font-size: 11pt;
    color: #444444;
}

div {
    margin: 0;
    padding: 0;
}

a {
    color: #444444;
}

a:visited {
    color: #444444;
}

h1, h2, h3, h4, h5 {
    color: #444444;
    line-height: 150%;
    font-weight: bold;
    font-family: 'DejaVu Sans', 'Lucida Grande', helvetica, sans-serif;
}

.heading {
    font-family: 'DejaVu Sans', 'Lucida Grande', helvetica, sans-serif;
}

code, pre {
    font-family: 'Lucida Console', 'DejaVu Sans Mono', Courier, monospace;
    border: 1px #eee solid;
    padding: 2px;
    background: #f7f7f7;
}

/*
 * Docbook content
 */

/*
 * Lists
 */

.variablelist dt {
    font-weight: bold;
}

.variablelist dt a {
    font-weight: normal;
}

/*
 * Examples, Figures
 */

pre {
    background-color: #f5f5f5;
    padding-top: 1em;
    padding-bottom: 1em;
    padding-left: 1.2em;
    padding-right: 1.2em;
    margin-top: 0.6em;
    margin-bottom: 0.6em;
    line-height: 120%;
}

.programlisting, .informalexample, .synopsis {
    border-left: solid #d0d0d0 0.5em;
    /*overflow-x: auto;*/
}

.screen {
    border-left: solid #d0d0d0 0.5em;
    /*overflow-x: auto;*/
}

.example, .figure, .table {
    margin-top: 0;
    margin-bottom: 1.8em;
}

.example .title, .figure .title, .table .title {
    white-space: nowrap;
    /*overflow-x: hidden;*/
    margin-top: 0;
    margin-bottom: 0.6em;
}

.example pre, .figure pre {
    margin: 0;
    margin-bottom: 0.6em;
}

.example-contents > p {
    display: block;
    margin: 0;
    margin-bottom: 0.2em;
}

.example-break, .figure-break {
    display: none
}

.exampleLocation {
    margin-top: 0.5em;
    padding-left: 1.7em;
    background: #FEFEDD;
    color: #777744;
}

.exampleLocation p {
    padding-top: 0.4em;
    padding-bottom: 0.4em;
}

.exampleLocation .emphasis em {
    font-style: normal;
    font-weight: bold;
}

.cmdsynopsis {
    font-family: Courier, monospace;
    margin-left: 2em;
}

/*
 * Tables
 */

.table table {
    border-collapse: collapse;
    font-size: 100%;
    min-width: 50%;
    border: solid #d0d0d0 1px;
    line-height: 150%;
}

.table table td {
    text-align: left;
    vertical-align: text-top;
    padding-left: 0.8em;
    padding-right: 0.8em;
    padding-top: 0.3em;
    padding-bottom: 0.3em;
}

.table table thead td {
    font-weight: bold;
    border-bottom: solid #d0d0d0 1px;
    background-color: #f2f2f2;
}

/*
 * Footnotes, notes, tips
 */

.footnote sup {
    vertical-align: baseline;
    font-size: 100%;
}

.note, .tip {
    padding-top: 0.7em;
    padding-bottom: 0.7em;
    padding-left: 1.2em;
    padding-right: 1.2em;
    margin-bottom: 1em;
    margin-top: 1em;
}

.note {
    background: #ecf3ea;
    border: solid #ccd3ca 1px;
}

.note .title {
    color: #5283a1;
}

.tip {
    float: right;
    clear: right;
    margin-left: 2em;
    margin-top: 1em;
    margin-bottom: 1em;
    max-width: 30%;
    width: 30%;
}

.tip { 
    background: #FAF7F5;
    border: solid #e2d2c9 1px;
}

.sidebar-border {
    float: right;
    clear: right;
    background: #FFFFFF;
    padding-left: 0em;
    margin-bottom: 1em;
    margin-top: 1em;
    max-width: 32%;
    width: 32%;
}
.sidebar {
    background: #FAF7F5;
    border: solid #e2d2c9 1px;
    margin-left: 1.2em;
    padding-left: 1.2em;
    padding-top: 0.7em;
    padding-bottom: 0.7em;
    padding-right: 1.2em;
}

.tip .title {
    color: #80614D;
}

.sidebar .title {
    color: #80614D;
}

/* Remove top margins on headings inside notes and tips */
.tip h1, .note h1
.tip h2, .note h2,
.tip h3, .note h3,
.tip h4, .note h4,
.tip h5, .note h5,
.tip h6, .note h6
{
   margin-top: 0;
}

/*
 * Code highlighting
 */
.hl-string, .hl-number, .hl-value { color: #51913f; }

.hl-keyword { font-weight: bold; }

.hl-comment, .hl-doccomment { color: #929292; }

.hl-annotation { color: #777744; }
