package org.jolokia.util;

/*
 * Copyright 2009-2013 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Enumeration holding the method which
 * lead to this request. This information to allows
 * the restriction of security policy based on the HTTP method.
 */
public enum HttpMethod {

    /**
     * POST request
     */
    POST("post"),

    /**
     * GET request
     */
    GET("get");

    private String method;

    HttpMethod(String pMethod) {
        method = pMethod;
    }

    public String getMethod() {
        return method;
    }
}
