package org.jolokia.handler;

/*
 * Copyright 2009-2013 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



import java.util.List;
import java.util.Map;
import java.util.Set;


public class OpenExecData implements OpenExecDataMXBean {

	public OpenWriteData compositeData(OpenWriteData data) {
        return data;
    }

	public OpenWriteData[] arrayData(OpenWriteData[] data) {
        return data;
    }

	public List listData(List<OpenWriteData> data) {
        return data;
    }

	public Map mapData(Map<String, OpenWriteData> data) {
        return data;
    }

	public Set opSetData(Set<OpenWriteData> data) {
        return data;
    }
	
	public OpenWriteData overloaded(OpenWriteData data) {

        return data;
    }

	public String overloaded(String data) {
        return data;
    }
	

}
