
/*
 * Copyright 2009-2013 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
body {
    margin-top: 2em;
    margin-bottom: 2em;
    margin-left: auto;
    margin-right: auto;
    border-left: 1px #eee solid;
    border-right: 1px #eee solid;
    padding: 0 30px;
    max-width: 1020px;
}

a {
    text-decoration: none;
}

a:link, a:visited {
  color: #820A0A;
}

a:active, a:hover {
  background: #E9E9E9;
}


p {
   /* ensure paragraphs remain close to their parent heading */
   margin-top: 0;
   margin-bottom: 1em;
}

h1, h2, h3, h4, h5 {
    /* ensure paragraphs remain close to their parent heading */
    margin-bottom: 0.2em;
    color: #820A0A;
}

h1 {
    font-size: 180%;
}

h2 {
    font-size: 180%;
}

h3 {
    font-size: 110%;
}

.chapter h1 {
    margin-bottom: 1.0em;
}

.appendix h1 {
    margin-bottom: 1.0em;
}

.section h2 {
    font-size: 130%;
    margin-top: 1.4em;
}

/*
 * Table of Contents
 */

.toc dl, .list-of-examples dl {
    margin-top: 0;
    margin-bottom: 0;
}

.book .toc > dl > dt {
    margin-top: 0.6em;
}

.book .toc, .book .list-of-examples {
    margin-top: 2em;
}

/*
 * HEADER/FOOTER
 */

.navheader {
    margin-bottom: 3em;
    padding-bottom: 1em;
    width: 100%;
    border-bottom: solid #d0d0d0 0.4em;
}

.navfooter {
    margin-top: 3em;
    padding-top: 1em;
    width: 100%;
    border-top: solid #d0d0d0 0.4em;
}

.navbar {
    color: #a2a2a2;
    font-size: 90%;
    width: 100%;
    text-align: right;
}

.navbar a {
    color: #444444;
}

.navbar span {
    padding-left: .8em;
    padding-right: .8em;
}

/**
 * TITLEPAGE
 */

.book > .titlepage {
    margin-top: 2.2em;
    margin-bottom: 2em;
}

.book .titlepage div.title {
    border: solid #d0d0d0 1px;
    background-color: #ffffff;
    margin-left: -30px;
    margin-right: -30px;
    padding-left: 1.8em;
    padding-right: 1.8em;
    margin-bottom: 2.2em;
}

.titlepage h1.title {
    font-size: 200%;
    margin-bottom: 0;
    color: #820A0A;
}

.titlepage h2.subtitle {
    font-size: 160%;
    font-style: italic;
    margin-top: 0;
    margin-bottom: 0;
    color: #4a9935;
}

.titlepage .releaseinfo {
    color: #666666;
    font-size: 120%;
    margin-top: 2em;
    margin-bottom: 2.6em;
}

.titlepage .author {
    color: #666666;
}

.titlepage .copyright {
    color: #a2a2a2;
    font-size: 90%;
}

.titlepage .legalnotice {
    color: #a2a2a2;
    font-size: 90%;
}

.literal, .userinput, .filename {
    white-space: nowrap;
}

/*
 * Single page html
 */
.book .chapter {
    margin-top: 4em;
}

div.mediaobject {
  text-align: center;
}
