package org.jolokia.jvmagent;

/*
 * Copyright 2009-2013 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.IOException;

import org.jolokia.jvmagent.JvmAgent;
import org.jolokia.test.util.EnvTestUtil;
import org.testng.annotations.Test;

/**
 * @author roland
 * @since 28.09.11
 */
public class JvmAgentTest {

    @Test
    public void premain() throws IOException {
        JvmAgent.premain("port=" + EnvTestUtil.getFreePort(), null);
        JvmAgent.agentmain("mode=stop", null);
    }

    @Test
    public void agentmain() throws IOException {
        JvmAgent.agentmain("mode=start,port=" + EnvTestUtil.getFreePort(), null);
        JvmAgent.agentmain("mode=stop", null);
    }

    @Test
    public void startException() throws IOException {
        int port = EnvTestUtil.getFreePort();
        JvmAgent.agentmain("port=" + port, null);
        try {
            JvmAgent.agentmain("port=" + port, null);
        } finally {
         JvmAgent.agentmain("mode=stop", null);
        }
    }

}
