package org.jolokia.converter.json.simplifier;

/*
 * Copyright 2009-2013 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.Date;
import java.util.Map;

import org.testng.annotations.Test;


/**
 * @author roland
 * @since Jul 2, 2010
 */
@Test(enabled = false)
public class TestSimplifier extends SimplifierExtractor<Date> {
    public TestSimplifier() {
        super(Date.class);
    }

    @Override
    void init(Map<String, AttributeExtractor<Date>> pStringAttributeExtractorMap) {
        Object[][] pAttrs = {
                { "millis", new AttributeExtractor<Date>() {
                    public Object extract(Date value) throws SkipAttributeException {
                        return value.getTime();
                    }
                }
                },
        };
        addExtractors(pAttrs);
    }
}
