package org.jolokia.util;
/*
 *
 * Copyright 2016 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Loghandler using java.util.logging with logger org.jolokia
 */
public class JulLogHandler implements LogHandler {

    private final Logger julilogger = Logger.getLogger("org.jolokia");

    public void debug(String message) {
        julilogger.fine(message);
    }

    public void info(String message) {
        julilogger.info(message);
    }

    public void error(String message, Throwable t) {
        julilogger.log(Level.WARNING, message, t);
    }
}
