package org.jolokia.converter.json.simplifier;

import javax.management.ObjectName;

/*
 * Copyright 2009-2013 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/**
 * Special deserialization for ObjectNames to their canonical format. It represents an
 * {@link ObjectName} as a JSON object with a single property, <code>objectName</code>, which
 * contains the canonical name.
 *
 * @author roland
 * @since Jul 27, 2009
 */
public class ObjectNameSimplifier extends SimplifierExtractor<ObjectName> {

    /**
     * Default constructor
     */
    public ObjectNameSimplifier() {
        super(ObjectName.class);
        addExtractor("objectName", new ObjectNameAttributeExtractor());
    }

    // ==================================================================================

    /** {@inheritDoc} */
    private static class ObjectNameAttributeExtractor implements AttributeExtractor<ObjectName> {
        /** {@inheritDoc} */
        public Object extract(ObjectName value)  {
            return value.getCanonicalName();
        }
    }
}