package org.jolokia.it;

/*
 * Copyright 2009-2013 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.jolokia.jmx.JsonMBean;

/**
 * @author roland
 * @since 27.01.13
 */
@JsonMBean
public class JsonChecking2 implements JsonChecking2MXBean {

    ComplexTestData data;

    public JsonChecking2() {
        data = new ComplexTestData();
    }

    public ComplexTestData getData() {
        return data;
    }

    public ComplexTestData execute(String[] args) {
        ComplexTestData data = new ComplexTestData();
        data.setStringArray(args);
        return data;
    }
}
