package org.jolokia.jvmagent.spring.config;

/*
 * Copyright 2009-2013 Roland Huss
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.jolokia.jvmagent.spring.SystemPropertyMode;
import org.testng.annotations.Test;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNull;

/**
 * @author roland
 * @since 01.01.13
 */
public class SystemPropertyModeTest {

    @Test
    public void basic() {
        assertEquals(SystemPropertyMode.fromMode("fallback"),SystemPropertyMode.MODE_FALLBACK);
        assertEquals(SystemPropertyMode.fromMode("NeVeR"),SystemPropertyMode.MODE_NEVER);
        assertNull(SystemPropertyMode.fromMode("Blub"));
        assertNull(SystemPropertyMode.fromMode(null));
    }
}
